/* ***************************************************************** */

/*	line.c		create a line drawing window */

/*	created by James Peterson, MCC */

/*	

 	Written on 5 Sept 1986

*/


/* ***************************************************************** */

#include <stdio.h>

/* missing types from C */
#define Boolean short 
#define true 1
#define false 0

/* ............................................................. */

/* window variables */

#include <X11/Xlib.h>

Boolean in_window /* are we in a window */ ;

Display *dpy;
Window window;
Boolean RedrawRequested = false /* when redraw requested */ ;


int     window_width, window_height;
float xdelta, ydelta;
float xscale = 1.0;
float yscale = 1.0;
float xmin = 0.0, ymin = 0.0;
float xmax = 1.0, ymax = 1.0;
#define EPS .0001



/* ............................................................. */

Boolean debug = false;

/* ............................................................. */

FILE *input;




/* ************************************************************ */
/*								*/
/*								*/
/*								*/
/* ************************************************************ */

/* ............................................................. */

ScanArgs(argc, argv)
int     argc;
char  **argv;
{
 /* first skip program name */
    argc--, argv++;

 /* get -flag options */
    while (argc > 0 && **argv == '-')
	{
	    char   *cp = *argv;
	    argc--, argv++;
	    switch (*++cp)
		{
		    case 'd': 
			    debug = true;
			    fprintf(stderr, "Debugging on.\n");
			    break;

		    default: 
			    fprintf(stderr, "usage: line [file]\n");
			    exit(1);
		}

	}

 /* other parameters, without the leading minus sign */
    if (argc > 0)
	{
	    input = fopen(*argv, "r");
	    if (input == NULL)
		{
		    fprintf(stderr, "can't open %s\n", *argv);
		    exit(1);
		}
	    if (debug) fprintf(stderr, "input is  %s\n", *argv);
	}
    else
	input = stdin;
}


/* ************************************************************ */
/*								*/
/*								*/
/*								*/
/* ************************************************************ */

char   *term;
char   *getenv ();
GC	realGC;

InitDisplay(argc, argv)
     int argc;
     char *argv[];
{
    XSetWindowAttributes xswa;
    long xsm = 0L;
    
 /* create new window */
    dpy = XOpenDisplay("");
    if (dpy == NULL)
	{
	    fprintf(stderr, "Error starting X\n");
	    exit(1);
	}
    window_width = 100;
    window_height = 100;
    xswa.background_pixel = WhitePixel(dpy, DefaultScreen(dpy));
    xsm |= CWBackPixel;
    
    
    window = XCreateWindow(dpy, RootWindow(dpy, DefaultScreen(dpy)),
			   0, 0,
			   window_width, window_height,
			   1,
			   0,
			   InputOutput, CopyFromParent,
			   xsm, &xswa);

    XSetStandardProperties(dpy, window,
			   argv[0], "line",
			   None, argv, argc, NULL);

    realGC = DefaultGC(dpy, DefaultScreen(dpy));
    
    XMapWindow(dpy, window);
    XSelectInput(dpy, window, ButtonPressMask|KeyPressMask);

    /* create new window and spawn process for it */
    /* if (fork() > 0)
		exit(0); */

    FlagRedraw();
}

/* ............................................................. */

FlagRedraw()
{
    RedrawRequested = true;
    if (debug)
	fprintf(stderr, "FlagRedraw\n");
}

/* ............................................................. */

DoRedraw()
{
    XWindowAttributes watt;
     
    RedrawRequested = false;
    XClearWindow(dpy, window);
    XGetWindowAttributes(dpy, window, &watt);
    window_width = watt.width;
    window_height = watt.height;
    
    if (debug)
	fprintf(stderr, "DoRedraw width %d height %d\n",
		window_width, window_height);
    if (window_width < 1 || window_height < 1)
	return;

	/* scale data */
	xdelta = (xmax - xmin); if (xdelta < EPS) xdelta = EPS;
	ydelta = (ymax - ymin); if (ydelta < EPS) ydelta = EPS;
	xscale = window_width/xdelta;
	yscale = window_height/ydelta;
	if (xscale < yscale) yscale = xscale;
	if (yscale < xscale) xscale = yscale;

 /* action part */
    DisplayData();
}

/* ............................................................. */

DoInput(ev)
    XButtonEvent *ev;
{
    int     c;
    int     Action,
            x,
            y;

 /* input is waiting on winin */
    if (debug)
	fprintf(stderr, "Read from winout ");
    MouseAction(ev->x, ev->y);
    DisplayData();
}

#include <X11/Xutil.h>
DoInputK(ev)
    XKeyEvent *ev;
{
    int     c;
    int     Action,
            x,
            y;
    char    cc[1];
    
    static XComposeStatus xcs;

    XLookupString(ev, cc, 1, NULL, &xcs);
    c = cc[0];

    /* input is waiting on winin */
    if (debug)
	fprintf(stderr, "Read from winout ");
    Keyboard(c);
    DisplayData();
}

/* ............................................................. */

#include <sys/time.h>


/* ............................................................. */

/* ************************************************************ */
/*								*/
/*								*/
/*								*/
/* ************************************************************ */

main(argc, argv)
int     argc;
char   *argv[];
{
    ScanArgs(argc, argv);
    InitData();
    InitDisplay(argc, argv);

    if (RedrawRequested)
	DoRedraw();
    while (true)
	{
	    XEvent xev;
	    
	    XNextEvent(dpy, &xev);

	    switch(xev.type) {
	    case ButtonPress:
		DoInput(&(xev.xbutton));
		break;
	    case KeyPress:
		DoInputK(&(xev.xkey));
		break;
	    case Expose:
		DoRedraw();
		break;
	    }
	}
}



/* ************************************************************ */
/*								*/
/*								*/
/*								*/
/* ************************************************************ */

/* for this application, we need a sequence of line segments,
	x and y coordinates, for two points */

/* we keep a linked list of points */

struct point { float x,y; };

struct segment
{
	struct segment *next;
	struct point from;
	struct point to;
};

struct segment *root;

long number;

InitData()
{
	int c,n;
	struct segment *p;

	number = 0;
	root = NULL;
	while ((c = getc(input)) != EOF)
	{
		ungetc(c,input);
		p = (struct segment *) malloc(sizeof(struct segment));
		if (p == NULL)
		   {
			fprintf(stderr,"no room for segment\n");
			return;
		   }
		n = fscanf(input,"%f %f %f %f",
			 &(p->from.x), &(p->from.y), &(p->to.x), &(p->to.y));
		if (debug) fprintf(stderr,"%d %f %f %f %f\n",n,
			 p->from.x, p->from.y, p->to.x, p->to.y);
		if (0 < n && n != 4)
			fprintf(stderr,"bad input (%d)\n",n);
		if (n != 4) break;

		/* adjust for y axis */
		p->to.y = 90 - p->to.y;
		p->from.y = 90 - p->from.y;

		p->next = root;
		root = p;
		number += 1;
		if (number % 10000 == 0)
			fprintf(stderr,"%d segments\n",number);
	}

	fprintf(stderr,"%d segments read\n",number);
	if (number <= 1) 
		{
		   fprintf(stderr,"Hard to plot one point\n");
		   exit();
		}

	xmax = xmin = root->to.x; 
	ymax = ymin = root->to.y; 
	for (p = root; p != NULL; p = p->next)
	{
		/* check for min, max */
		if (xmin > p->to.x) xmin = p->to.x;
		if (xmin > p->from.x) xmin = p->to.x;
		if (ymin > p->to.y) ymin = p->to.y;
		if (ymin > p->from.y) ymin = p->to.y;
		if (xmax < p->to.x) xmax = p->to.x;
		if (xmax < p->from.x) xmax = p->to.x;
		if (ymax < p->to.y) ymax = p->to.y;
		if (ymax < p->from.y) ymax = p->to.y;
	}
	fprintf(stderr,"%8.2f < x < %8.2f, %8.2f < y < %8.2f\n",
			xmin, xmax, ymin, ymax );

}

DisplayData()
{
	struct segment *p;
	int x1,y1,x2,y2,i;
	XSegment *xsgs;

	xsgs = (XSegment *)malloc(sizeof(XSegment)*number);
	
	XClearWindow(dpy, window);
	fprintf(stderr,"xs,ys,xm,ym=%f,%f,%f,%f\n",
		xscale, yscale,
		xmin, ymin);

	/* draw scaled line segments */
	for (p = root,i=0; p != NULL; p = p->next)
	     {
		 xsgs[i].x1 = xscale * (p->from.x - xmin);
		 xsgs[i].y1 = yscale * (p->from.y - ymin);
		 xsgs[i].x2 = xscale * (p->to.x - xmin);
		 xsgs[i].y2 = yscale * (p->to.y - ymin);
		 i++;
	     }
	XDrawSegments(dpy, window, realGC, xsgs, number);
	XFlush(dpy);
	free(xsgs);
}

MouseAction(x,y)
int x,y;
{
  /* recenter */
  xmin += (x - window_width/2)/xscale;
  ymin += (y - window_height/2)/yscale;
}

Keyboard(c)
char c;
{
    switch(c) {
    case 'z':			/* zoom */
	xscale = 2*xscale;
	yscale = 2*yscale;
	break;
    case 's':			/* shrink */
	xscale = xscale/2;
	yscale = yscale/2;
	break;
    case 'q':
	exit(0);
	break;
    case 'l':
	xmin = 0;
	ymin = 0;
	break;
    }
}
